from PIL.ImageFont import truetype
from os.path import isdir, join
from os import listdir
from re import compile


pattern = compile('[^a-z0-9]+')

fonts = None
cache = None


def loadFonts(directories):
    """ load available fonts from the given list of directories """
    global cache, fonts
    cache = {}
    fonts = {}
    for dir in directories:
        for name in listdir(dir):
	    path = join(dir, name)
	    if isdir(path): continue
	    try: font = truetype(path, 10)
	    except: continue
	    family, style = font.getname()
	    title = '%s (%s)' % (family, style.lower())
	    key = pattern.sub(' ', title.lower())
	    key = key.strip().replace(' ', '-')
	    fonts[key] = {
	        'title': title,
		'family': family,
		'style': style,
		'path': path,
	    }


def getFontNames():
    """ return ids and names of all available fonts """
    names = [ (key, info.get('title')) for key, info in fonts.items() ]
    names.sort(lambda a,b: cmp(a[1], b[1]))
    return tuple(names)


def getFont(name, size=10):
    """ return an initialized pil font of the given size """
    key = (name, size)
    if cache.has_key(key):
        return cache.get(key)
    info = fonts.get(name)
    font = truetype(info.get('path'), size)
    cache[key] = font
    return font

