from StringIO import StringIO
from PIL.ImageEnhance import Brightness
from PIL.Image import new, open as popen, composite
from PIL.ImageFont import truetype
from PIL.ImageDraw import Draw
from sys import argv, exit
from os.path import isdir, join
from os import listdir


def reduceOpacity(im, opacity):
    """ returns an image with reduced opacity """
    assert opacity >= 0 and opacity <= 1
    if im.mode != 'RGBA':
        im = im.convert('RGBA')
    else:
        im = im.copy()
    alpha = im.split()[3]
    alpha = Brightness(alpha).enhance(opacity)
    im.putalpha(alpha)
    return im


def convertToJpeg(img, size=None, **args):
    """ convert the image to jpeg format and return the data """
    target = StringIO()
    img = img.convert('RGB')
    if size: img.resize(size)
    img.save(target, "JPEG", optimize=1, **args)
    target.seek(0)
    return target.getvalue()


def watermark(im, mark, position, opacity=1):
    """ adds a watermark to an image """
    if opacity < 1:
        mark = reduceOpacity(mark, opacity)
    if not im.mode == 'RGBA':
        im = im.convert('RGBA')
    # create a transparent layer the size of the image and draw the
    # watermark in that layer.
    layer = new('RGBA', im.size, (0,0,0,0))
    if position == 'tile':
        for y in range(0, im.size[1], mark.size[1]):
            for x in range(0, im.size[0], mark.size[0]):
                layer.paste(mark, (x, y))
    elif position == 'scale':
        # scale, but preserve the aspect ratio
        ratio = min(float(im.size[0]) / mark.size[0],
	            float(im.size[1]) / mark.size[1])
        w = int(mark.size[0] * ratio)
        h = int(mark.size[1] * ratio)
        mark = mark.resize((w, h))
        layer.paste(mark, ((im.size[0] - w) / 2, (im.size[1] - h) / 2))
    else:
        layer.paste(mark, position)
    # composite the watermark with the layer
    return composite(layer, im, layer)


def imprint(im, text='', font=None, color=None, margin=(10, -10), opacity=1):
    """ imprints an image with the indicated text """
    if not im.mode == 'RGBA':
        im = im.convert('RGBA')
    layer = new('RGBA', im.size, (0,0,0,0))
    draw = Draw(layer)
    size = draw.textsize(text, font=font)
    xpos, ypos = margin
    if xpos < 0: xpos += im.size[0] - size[0]
    if ypos < 0: ypos += im.size[1] - size[1]
    draw.text((xpos, ypos), text, font=font, fill=color)
    if opacity < 1:
        layer = reduceOpacity(layer, opacity)
    return composite(layer, im, layer)

